import { map } from "../iters";
import { listMultimapPush } from "../utils";
/**
 * get a mapping from a the children of a set of nodes to their unique parents
 */
export function getParents(parentNodes) {
    const parents = new Map();
    for (const par of parentNodes) {
        for (const child of par.ichildren()) {
            listMultimapPush(parents, child, par);
        }
    }
    return parents;
}
/**
 * get a mapping from a the children of a set of nodes to their parents with counts
 */
export function getParentCounts(parentNodes) {
    const parents = new Map();
    for (const par of parentNodes) {
        for (const [child, count] of par.ichildrenCounts()) {
            listMultimapPush(parents, child, [par, count]);
        }
    }
    return parents;
}
/** convert a dag into a dot string */
export function dot(dag, id) {
    const links = [
        ...map(dag.ilinks(), ({ source, target }) => `    "${id(source)}" -> "${id(target)}"`)
    ];
    return `digraph {\n${links.join("\n")}\n}`;
}
