/**
 * Stopgap for esnext iterable features
 *
 * @module
 */
export function* entries(iter) {
    let index = 0;
    for (const element of iter) {
        yield [index++, element];
    }
}
export function* flatMap(iter, callback) {
    for (const [index, element] of entries(iter)) {
        yield* callback(element, index);
    }
}
export function reduce(iter, callback, initialValue) {
    const call = callback;
    let accumulator = initialValue;
    for (const [index, element] of entries(iter)) {
        accumulator = call(accumulator, element, index);
    }
    return accumulator;
}
export function* map(iter, callback) {
    for (const [index, element] of entries(iter)) {
        yield callback(element, index);
    }
}
export function* filter(iter, callback) {
    for (const [index, element] of entries(iter)) {
        if (callback(element, index)) {
            yield element;
        }
    }
}
export function some(iter, callback) {
    for (const [index, element] of entries(iter)) {
        if (callback(element, index)) {
            return true;
        }
    }
    return false;
}
export function every(iter, callback) {
    return !some(iter, (e, i) => !callback(e, i));
}
export function* reverse(arr) {
    for (let i = arr.length; i != 0;) {
        yield arr[--i];
    }
}
